/* YUI datatable overrides */

/* results matrix remove 'button' */
div.yui-dt-col-_remove.yui-dt-liner
{
    text-align: center;
    color: #7A0019;
    font-size: 90%;
    cursor:pointer;
}

/* div.yui-dt-col-_remove:hover, */
div.yui-dt-col-_remove.hover
{
    color: #000;
    text-decoration: underline;
}

.yui-skin-sam .yui-dt-table caption {
    /* padding-bottom:1em; */
    padding: 0.5em;
    text-align:left;
    border: 1px solid #444;
    font-weight: bold;

}

.yui-skin-sam .yui-pg-container,
.yui-skin-sam .yui-dt-paginator {
    display:block;
    margin:6px 0;
    white-space:nowrap;
    padding: 4px;
}

.yui-skin-sam .yui-pg-first,
.yui-skin-sam .yui-pg-last,
.yui-skin-sam .yui-pg-current-page,
.yui-skin-sam .yui-dt-first,
.yui-skin-sam .yui-dt-paginator .yui-dt-last,
.yui-skin-sam .yui-dt-paginator .yui-dt-selected {
    padding:2px 6px;
}
.yui-skin-sam a.yui-pg-first,
.yui-skin-sam a.yui-pg-previous,
.yui-skin-sam a.yui-pg-next,
.yui-skin-sam a.yui-pg-last,
.yui-skin-sam a.yui-pg-page,
.yui-skin-sam .yui-dt-paginator a.yui-dt-first,
.yui-skin-sam .yui-dt-paginator a.yui-dt-last {
    text-decoration:none;
}
.yui-skin-sam .yui-dt-paginator .yui-dt-previous,
.yui-skin-sam .yui-dt-paginator .yui-dt-next {
    display:none;
}
.yui-skin-sam a.yui-pg-page,
.yui-skin-sam a.yui-dt-page {
    border:1px solid #7a0019;
    padding:2px 6px;
    text-decoration:none;
    background-color:#fff;
}
.yui-skin-sam .yui-pg-current-page,
.yui-skin-sam .yui-dt-selected {
    border:1px solid #000;
    /* background-color:#B2D2FF; */  /* YUI blue */
    background-color: #eee;
}
.yui-skin-sam .yui-pg-pages {
    margin-left:1ex;
    margin-right:1ex;
}
.yui-skin-sam .yui-pg-page {
    margin-right:1px;
    margin-left:1px;
}
.yui-skin-sam .yui-pg-first,
.yui-skin-sam .yui-pg-previous {
    margin-right:3px;
    border: 1px solid #aaa;
    padding: 2px 4px 2px 4px;
    background-color: #fff;
}
.yui-skin-sam .yui-pg-next,
.yui-skin-sam .yui-pg-last {
    margin-left:3px;
    border: 1px solid #aaa;
    padding: 2px 4px 2px 4px;
    background-color: #fff;
}

.yui-skin-sam a.yui-pg-first,
.yui-skin-sam a.yui-pg-previous,
.yui-skin-sam a.yui-pg-next,
.yui-skin-sam a.yui-pg-last
{
    border: 1px solid #7a0019;
    
}

.yui-skin-sam .yui-pg-current,
.yui-skin-sam .yui-pg-rpp-options {
    margin-right:1em;
    margin-left:1em;
    border: 1px solid #ccc;
    padding: 4px;
    background-color: #fff;
}

/* sortable column headers should be colorful */
.yui-skin-sam .yui-dt th div a
{
    color: #7A0019;
    border-bottom: none;
}

.yui-skin-sam .yui-dt th div:hover
{
    color: #7A0019;
    background-color: #ffdd77;
}

/* datatable */
.yui-skin-sam .yui-dt-body { 
    cursor:pointer; /* when rows are selectable */
}

.yui-skin-sam .yui-dt-data {
    
}

/* fix OSX gecko bug per http://developer.yahoo.com/yui/container/ */
.yui-panel-container.hide-scrollbars #addListPanel .yui-dt-bd,
.yui-panel-container.hide-scrollbars #links_panel_container .yui-dt-bd,
.yui-panel-container.hide-scrollbars #daily_schedule_panel .yui-dt-bd
{
    /* Hide scrollbars by default for Gecko on OS X */
    overflow: hidden;
}

.yui-panel-container.show-scrollbars #addListPanel .yui-dt-bd,
.yui-panel-container.show-scrollbars #links_panel_container .yui-dt-bd,
.yui-panel-container.show-scrollbars #daily_schedule_panel .yui-dt-bd
{
    /* Show scrollbars for Gecko on OS X when the Panel is visible  */
    overflow: auto;
}
