
/* tabview */

/* .yui-navset defaults to .yui-navset-top */
.yui-skin-sam .yui-navset .yui-nav,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav { /* protect nested tabviews from other orientations */
    border:solid #7A0019; /* color between tab list and content */
    border-width:0 0 5px;
    Xposition:relative;
    zoom:1;
}

.yui-skin-sam .yui-navset .yui-nav li,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav li {
    margin:0 0.16em 0 0; /* space between tabs */
    padding:1px 0 0; /* gecko: make room for overflow */
    zoom:1;
}

.yui-skin-sam .yui-navset .yui-nav .selected,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav .selected { 
    margin:0 0.16em -1px 0; /* for overlap */
}

.yui-skin-sam .yui-navset .yui-nav a,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav a {
    /*
    background:#d8d8d8 url(../../../../assets/skins/sam/sprite.png) repeat-x;
    */
    background: #ffffff;
    border:solid #7A0019;
    border-width:0 1px;
    color:#000;
    text-decoration:none;
}

.yui-skin-sam .yui-navset .yui-nav a em,
.yui-skin-sam .yui-navset .yui-navset-top .yui-nav a em {
    border:solid #7A0019;
    border-width:1px 0 0;
    cursor:hand;
    padding:0.25em .75em;
    left:0; right: 0; bottom: 0; /* protect from other orientations */
    top:-1px; /* for 1px rounded corners */
    position:relative;
}

.yui-skin-sam .yui-navset .yui-nav .selected a,
.yui-skin-sam .yui-navset .yui-nav .selected a:focus, /* no focus effect for selected */
.yui-skin-sam .yui-navset .yui-nav .selected a:hover { /* no hover effect for selected */
    /* background:#2647a0 url(../../../../assets/skins/sam/sprite.png) repeat-x left -1400px; *//* selected tab background */
    background: #ffdd77;
    color:#7A0019;
}

.yui-skin-sam .yui-navset .yui-nav a:hover,
.yui-skin-sam .yui-navset .yui-nav a:focus {
    /* background:#bfdaff url(../../../../assets/skins/sam/sprite.png) repeat-x left -1300px; *//* selected tab background */
    background: #ffdd77;
    outline:0;
}

.yui-skin-sam .yui-navset .yui-nav .selected a em {
    padding:0.35em 0.75em; /* raise selected tab */
}

.yui-skin-sam .yui-navset .yui-nav .selected a,
.yui-skin-sam .yui-navset .yui-nav .selected a em {
    border-color:#111111; /* selected tab border color */
}

.yui-skin-sam .yui-navset .yui-content {
    background:#ffffff; /* content background color */
}

.yui-skin-sam .yui-navset .yui-content,
.yui-skin-sam .yui-navset .yui-navset-top .yui-content {
    border:1px solid #7A0019; /* content border */
    border-top-color:#7A0019; /* different border color */
    padding:1em; /* content padding */
}
