package CatalystX::CRUD::YUI::TT::crud;
use strict;

our $VERSION = '0.031';

=head1 NAME

CatalystX::CRUD::YUI::TT::crud - Template crud snippets

=head1 DESCRIPTION

This class is a placeholder for VERSION and contains documentation only.

See the description for new() in CatalystX::CRUD::YUI::View for how
to alter the behaviour of the .tt files in CatalystX::CRUD::YUI::TT::crud.

=cut

1;

__END__

=head1 AUTHOR

Peter Karman, C<< <karman@cpan.org> >>

=head1 BUGS

Please report any bugs or feature requests to
C<bug-catalystx-crud-yui@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.  I will be notified, and then you'll automatically be
notified of progress on your bug as I make changes.

=head1 ACKNOWLEDGEMENTS

The Minnesota Supercomputing Institute C<< http://www.msi.umn.edu/ >>
sponsored the development of this software.

=head1 COPYRIGHT & LICENSE

Copyright 2008 by the Regents of the University of Minnesota.


This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut


